import { deleteApp } from '@firebase/app';
import { getApp, getApps, initializeApp } from 'firebase/app';
import { getMessaging, getToken } from 'firebase/messaging';
import { FileStream } from './FileStream';
export class FcmMultipleProjectService {
    constructor() {
        this.fileStream = new FileStream();
        console.error('init FcmMultipleProjectService');
        this.init();
    }
    test() {
        // this.add(firebaseConfig);
        // firebaseConfig.firebaseAppName = 'third';
        // this.add(firebaseConfig);
        // firebaseConfig.firebaseAppName = 'forth';
        // this.add(firebaseConfig);
        // todo nabu check how to work with add method if equal config but different name
    }
    init() {
        const firebaseConfigs = this.fileStream.readStorage();
        firebaseConfigs.forEach(firebaseConfig => this.initializeApp(firebaseConfig));
    }
    add(firebaseConfig) {
        this.hasRights(firebaseConfig, () => {
            this.fileStream.writeStorage(firebaseConfig);
            this.remove(firebaseConfig);
            this.initializeApp(firebaseConfig);
        }, () => {
            console.error('hasRights error');
        });
    }
    clean() {
        const defaultFirebaseApp = this.getDefaultInstance();
        getApps()
            .filter(firebasApp => firebasApp.name !== defaultFirebaseApp.name)
            .forEach(firebaseApp => deleteApp(firebaseApp));
        this.fileStream.writeEmpty();
    }
    remove(firebaseConfig) {
        try {
            const firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
            console.error('InstanceApp has exist: ', firebaseConfig.firebaseAppName);
            if (firebaseApp.name === this.getDefaultInstance().name) {
                //default
                return;
            }
            deleteApp(firebaseApp).then();
        }
        catch (e) {
            console.error('InstanceApp not found:', firebaseConfig.firebaseAppName);
        }
    }
    getToken(firebaseConfig, apply, reject) {
        const firebaseMessaging = this.getFirebaseMessaging(firebaseConfig);
        getToken(firebaseMessaging)
            .then((currentToken) => {
            console.error('currentToken', currentToken);
            apply(currentToken);
        })
            .catch((err) => {
            console.error('An error occurred while retrieving token. ', err);
            reject(err);
        });
    }
    getDefaultInstance() {
        return getApp();
    }
    getInstance(firebaseAppName) {
        return getApp(firebaseAppName);
    }
    getFirebaseMessaging(firebaseConfig) {
        const firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
        return getMessaging(firebaseApp);
    }
    createFirebaseConfig(obj) {
        const firebaseConfig = obj;
        // used projectId because projectId has unique value from firebase
        firebaseConfig.firebaseAppName = firebaseConfig.projectId;
        return firebaseConfig;
    }
    hasRights(firebaseConfig, apply, reject) {
        if (!(firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.firebaseAppName) || !(firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.projectId)) {
            reject(null);
        }
        else {
            apply(null);
        }
    }
    initializeApp(firebaseConfig) {
        let firebaseApp;
        try {
            firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
            console.error('InstanceApp has exist: ', firebaseConfig.firebaseAppName);
        }
        catch (e) {
            console.error('InstanceApp not found:', firebaseConfig.firebaseAppName);
            const options = this.createFirebaseOptions(firebaseConfig);
            firebaseApp = initializeApp(options, firebaseConfig.firebaseAppName);
        }
        return firebaseApp;
    }
    createFirebaseOptions(firebaseConfig) {
        const firebaseOptions = {
            apiKey: firebaseConfig.apiKey,
            authDomain: undefined,
            databaseURL: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.databaseUrl,
            projectId: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.projectId,
            storageBucket: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.storageBucket,
            messagingSenderId: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.gcmSenderId,
            appId: firebaseConfig.applicationId,
            measurementId: firebaseConfig.measurementId,
        };
        return firebaseOptions;
    }
}
//# sourceMappingURL=fcm-multiple-project.service.js.map