import { WebPlugin } from '@capacitor/core';
import { FCM_EVENT_TOKEN_ERROR, FCM_EVENT_TOKEN_SUCCESS } from './multiple/FirebaseConfig';
import { FcmMultipleProjectService } from './multiple/fcm-multiple-project.service';
export class FCMMultipleProjectWeb extends WebPlugin {
    /**
     * The Singleton's constructor should always be private to prevent direct
     * construction calls with the `new` operator.
     */
    constructor() {
        super();
        this.fcmMultipleProject = new FcmMultipleProjectService();
        console.error('FCMMultipleProjectWeb');
    }
    static getInstance() {
        if (!FCMMultipleProjectWeb.instance) {
            FCMMultipleProjectWeb.instance = new FCMMultipleProjectWeb();
        }
        return FCMMultipleProjectWeb.instance;
    }
    async echo(options) {
        console.log('ECHO', options);
        return options;
    }
    add(firebaseConfig) {
        return new Promise((resolve, reject) => {
            const originFirebaseConfig = this.fcmMultipleProject.createFirebaseConfig(firebaseConfig);
            this.fcmMultipleProject.hasRights(originFirebaseConfig, () => {
                this.fcmMultipleProject.add(originFirebaseConfig);
                this.fcmMultipleProject.getToken(originFirebaseConfig, (token) => {
                    const firebaseToken = {
                        token,
                        firebaseAppName: originFirebaseConfig.firebaseAppName,
                    };
                    this.notifyListeners(FCM_EVENT_TOKEN_SUCCESS, firebaseToken);
                }, (error) => {
                    const firebaseTokenError = {
                        error,
                        firebaseAppName: originFirebaseConfig.firebaseAppName,
                    };
                    this.notifyListeners(FCM_EVENT_TOKEN_ERROR, firebaseTokenError);
                });
                resolve();
            }, () => {
                reject('No valid permission FirebaseConfig');
            });
            resolve(null);
        });
    }
    clean() {
        this.fcmMultipleProject.clean();
        return;
    }
}
//# sourceMappingURL=web.js.map