'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var core = require('@capacitor/core');
var app$1 = require('@firebase/app');
var app = require('firebase/app');
var messaging = require('firebase/messaging');

const FCM_EVENT_TOKEN_SUCCESS = 'multipleToken';
const FCM_EVENT_TOKEN_ERROR = 'multipleTokenError';

const FCMMultipleProject = core.registerPlugin('FCMMultipleProject', {
    web: () => Promise.resolve().then(function () { return web; }).then(m => {
        return m.FCMMultipleProjectWeb.getInstance();
    }),
});

var StorageNameEnum;
(function (StorageNameEnum) {
    StorageNameEnum["FCM_LOCAL_"] = "FCM_LOCAL_";
})(StorageNameEnum || (StorageNameEnum = {}));
var StorageLocalKeyEnum;
(function (StorageLocalKeyEnum) {
    StorageLocalKeyEnum["FCM_APPS"] = "FCM_APPS";
})(StorageLocalKeyEnum || (StorageLocalKeyEnum = {}));
class StorageLocal {
    constructor() {
        this.cache = new Map();
    }
    get(storageLocalKeyEnum) {
        const key = StorageNameEnum.FCM_LOCAL_ + storageLocalKeyEnum;
        let fromCache = this.cache.get(key);
        if (!!fromCache === false) {
            const value = localStorage.getItem(key);
            if (value == null || value === 'undefined' || value === undefined || value == '') {
                return null;
            }
            try {
                fromCache = JSON.parse(value);
            }
            catch (e) {
                fromCache = undefined;
                console.error('StorageLocalService need to refactor: ', e);
            }
            this.cache.set(key, fromCache);
        }
        return fromCache;
    }
    set(storageLocalKeyEnum, value) {
        const key = StorageNameEnum.FCM_LOCAL_ + storageLocalKeyEnum;
        this.cache.set(key, value);
        localStorage.setItem(key, JSON.stringify(value));
    }
    remove(storageLocalKeyEnum) {
        const key = StorageNameEnum.FCM_LOCAL_ + storageLocalKeyEnum;
        this.cache.delete(key);
        localStorage.removeItem(key);
    }
    removeAll() {
        this.cache.clear();
        const arr = []; // Array to hold the keys
        // Iterate over localStorage and insert the keys that meet the condition into arr
        for (let i = 0; i < localStorage.length; i++) {
            if (localStorage.key(i).substring(0, StorageNameEnum.FCM_LOCAL_.length) === StorageNameEnum.FCM_LOCAL_) {
                arr.push(localStorage.key(i));
            }
        }
        // Iterate over arr and remove the items by key
        // tslint:disable-next-line:prefer-for-of
        arr.forEach((item) => {
            localStorage.removeItem(item);
        });
    }
    setFCMApps(value) {
        this.set(StorageLocalKeyEnum.FCM_APPS, value);
    }
    getFCMApps() {
        return this.get(StorageLocalKeyEnum.FCM_APPS) || [];
    }
}
class FileStream {
    constructor() {
    }
    readStorage() {
        return FileStream.storageLocal.getFCMApps();
    }
    writeEmpty() {
        FileStream.storageLocal.setFCMApps([]);
    }
    writeStorage(firebaseConfig) {
        let firebaseConfigs = FileStream.storageLocal.getFCMApps();
        firebaseConfigs = firebaseConfigs
            .filter(config => {
            return config.firebaseAppName !== firebaseConfig.firebaseAppName;
        })
            .filter(config => {
            return config.projectId !== firebaseConfig.projectId;
        });
        firebaseConfigs.push(firebaseConfig);
        FileStream.storageLocal.setFCMApps(firebaseConfigs);
    }
}
FileStream.storageLocal = new StorageLocal();

class FcmMultipleProjectService {
    constructor() {
        this.fileStream = new FileStream();
        console.error('init FcmMultipleProjectService');
        this.init();
    }
    test() {
        // this.add(firebaseConfig);
        // firebaseConfig.firebaseAppName = 'third';
        // this.add(firebaseConfig);
        // firebaseConfig.firebaseAppName = 'forth';
        // this.add(firebaseConfig);
        // todo nabu check how to work with add method if equal config but different name
    }
    init() {
        const firebaseConfigs = this.fileStream.readStorage();
        firebaseConfigs.forEach(firebaseConfig => this.initializeApp(firebaseConfig));
    }
    add(firebaseConfig) {
        this.hasRights(firebaseConfig, () => {
            this.fileStream.writeStorage(firebaseConfig);
            this.remove(firebaseConfig);
            this.initializeApp(firebaseConfig);
        }, () => {
            console.error('hasRights error');
        });
    }
    clean() {
        const defaultFirebaseApp = this.getDefaultInstance();
        app.getApps()
            .filter(firebasApp => firebasApp.name !== defaultFirebaseApp.name)
            .forEach(firebaseApp => app$1.deleteApp(firebaseApp));
        this.fileStream.writeEmpty();
    }
    remove(firebaseConfig) {
        try {
            const firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
            console.error('InstanceApp has exist: ', firebaseConfig.firebaseAppName);
            if (firebaseApp.name === this.getDefaultInstance().name) {
                //default
                return;
            }
            app$1.deleteApp(firebaseApp).then();
        }
        catch (e) {
            console.error('InstanceApp not found:', firebaseConfig.firebaseAppName);
        }
    }
    getToken(firebaseConfig, apply, reject) {
        const firebaseMessaging = this.getFirebaseMessaging(firebaseConfig);
        messaging.getToken(firebaseMessaging)
            .then((currentToken) => {
            console.error('currentToken', currentToken);
            apply(currentToken);
        })
            .catch((err) => {
            console.error('An error occurred while retrieving token. ', err);
            reject(err);
        });
    }
    getDefaultInstance() {
        return app.getApp();
    }
    getInstance(firebaseAppName) {
        return app.getApp(firebaseAppName);
    }
    getFirebaseMessaging(firebaseConfig) {
        const firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
        return messaging.getMessaging(firebaseApp);
    }
    createFirebaseConfig(obj) {
        const firebaseConfig = obj;
        // used projectId because projectId has unique value from firebase
        firebaseConfig.firebaseAppName = firebaseConfig.projectId;
        return firebaseConfig;
    }
    hasRights(firebaseConfig, apply, reject) {
        if (!(firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.firebaseAppName) || !(firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.projectId)) {
            reject(null);
        }
        else {
            apply(null);
        }
    }
    initializeApp(firebaseConfig) {
        let firebaseApp;
        try {
            firebaseApp = this.getInstance(firebaseConfig.firebaseAppName);
            console.error('InstanceApp has exist: ', firebaseConfig.firebaseAppName);
        }
        catch (e) {
            console.error('InstanceApp not found:', firebaseConfig.firebaseAppName);
            const options = this.createFirebaseOptions(firebaseConfig);
            firebaseApp = app.initializeApp(options, firebaseConfig.firebaseAppName);
        }
        return firebaseApp;
    }
    createFirebaseOptions(firebaseConfig) {
        const firebaseOptions = {
            apiKey: firebaseConfig.apiKey,
            authDomain: undefined,
            databaseURL: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.databaseUrl,
            projectId: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.projectId,
            storageBucket: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.storageBucket,
            messagingSenderId: firebaseConfig === null || firebaseConfig === void 0 ? void 0 : firebaseConfig.gcmSenderId,
            appId: firebaseConfig.applicationId,
            measurementId: firebaseConfig.measurementId,
        };
        return firebaseOptions;
    }
}

class FCMMultipleProjectWeb extends core.WebPlugin {
    /**
     * The Singleton's constructor should always be private to prevent direct
     * construction calls with the `new` operator.
     */
    constructor() {
        super();
        this.fcmMultipleProject = new FcmMultipleProjectService();
        console.error('FCMMultipleProjectWeb');
    }
    static getInstance() {
        if (!FCMMultipleProjectWeb.instance) {
            FCMMultipleProjectWeb.instance = new FCMMultipleProjectWeb();
        }
        return FCMMultipleProjectWeb.instance;
    }
    async echo(options) {
        console.log('ECHO', options);
        return options;
    }
    add(firebaseConfig) {
        return new Promise((resolve, reject) => {
            const originFirebaseConfig = this.fcmMultipleProject.createFirebaseConfig(firebaseConfig);
            this.fcmMultipleProject.hasRights(originFirebaseConfig, () => {
                this.fcmMultipleProject.add(originFirebaseConfig);
                this.fcmMultipleProject.getToken(originFirebaseConfig, (token) => {
                    const firebaseToken = {
                        token,
                        firebaseAppName: originFirebaseConfig.firebaseAppName,
                    };
                    this.notifyListeners(FCM_EVENT_TOKEN_SUCCESS, firebaseToken);
                }, (error) => {
                    const firebaseTokenError = {
                        error,
                        firebaseAppName: originFirebaseConfig.firebaseAppName,
                    };
                    this.notifyListeners(FCM_EVENT_TOKEN_ERROR, firebaseTokenError);
                });
                resolve();
            }, () => {
                reject('No valid permission FirebaseConfig');
            });
            resolve(null);
        });
    }
    clean() {
        this.fcmMultipleProject.clean();
        return;
    }
}

var web = /*#__PURE__*/Object.freeze({
    __proto__: null,
    FCMMultipleProjectWeb: FCMMultipleProjectWeb
});

exports.FCMMultipleProject = FCMMultipleProject;
exports.FCM_EVENT_TOKEN_ERROR = FCM_EVENT_TOKEN_ERROR;
exports.FCM_EVENT_TOKEN_SUCCESS = FCM_EVENT_TOKEN_SUCCESS;
//# sourceMappingURL=plugin.cjs.js.map
